@@ -127,7 +127,7 @@ def step(state: State, action: jnp.ndarray) -> State:
     return jax.lax.cond(
         _is_all_off(state.board, state.turn),
         lambda: _winning_step(state),
-        lambda: _no_winning_step(state),
+        lambda: _no_winning_step(state, action),
     )
 
 
@@ -182,26 +182,22 @@ def _winning_step(
     return state.replace(reward=reward)  # type: ignore
 
 
-def _no_winning_step(
-    state: State,
-) -> State:
+def _no_winning_step(state: State, action: jnp.ndarray) -> State:
     """
     勝利者がいない場合のstep, ターン終了の条件を満たせばターンを変更する.
     """
-    return _change_until_legal(state)
-
-
-def _change_until_legal(state: State) -> State:
-    """
-    行動可能なplayerが出るまでturnを変え続ける.
-    """
-    return jax.lax.while_loop(_is_turn_end, _change_turn, state)
+    return jax.lax.cond(
+        (_is_turn_end(state) | (action // 6 == 0)),
+        lambda: _change_turn(state),
+        lambda: state,
+    )
 
 
 def _update_by_action(state: State, action: jnp.ndarray) -> State:
     """
     行動を受け取って状態をupdate
     """
+    is_no_op = action // 6 == 0
     rng = state.rng
     curr_player: jnp.ndarray = state.curr_player
     terminated: jnp.ndarray = state.terminated
@@ -213,17 +209,21 @@ def _update_by_action(state: State, action: jnp.ndarray) -> State:
     legal_action_mask: jnp.ndarray = _legal_action_mask(
         board, state.turn, playable_dice
     )
-    return State(  # type: ignore
-        curr_player=curr_player,
-        rng=rng,
-        terminated=terminated,
-        board=board,
-        turn=state.turn,
-        dice=state.dice,
-        playable_dice=playable_dice,
-        played_dice_num=played_dice_num,
-        legal_action_mask=legal_action_mask,
-    )
+    return jax.lax.cond(
+        is_no_op,
+        lambda: state,
+        lambda: State(  # type: ignore
+            curr_player=curr_player,
+            rng=rng,
+            terminated=terminated,
+            board=board,
+            turn=state.turn,
+            dice=state.dice,
+            playable_dice=playable_dice,
+            played_dice_num=played_dice_num,
+            legal_action_mask=legal_action_mask,
+        ),
+    )  # no-opの時はupdateしない
 
 
 def _make_init_board() -> jnp.ndarray:
@@ -267,9 +267,7 @@ def _is_turn_end(state: State) -> bool:
     """
     play可能なサイコロ数が0の場合ないしlegal_actionがない場合交代
     """
-    return (state.playable_dice.sum() == -4) | (
-        state.legal_action_mask.sum() == 0
-    )  # type: ignore
+    return state.playable_dice.sum() == -4  # type: ignore
 
 
 def _change_turn(state: State) -> State:
@@ -280,7 +278,7 @@ def _change_turn(state: State) -> State:
     board: jnp.ndarray = state.board
     turn: jnp.ndarray = -1 * state.turn  # turnを変える
     curr_player: jnp.ndarray = (state.curr_player + 1) % 2
-    terminated: jnp.ndarray = FALSE
+    terminated: jnp.ndarray = state.terminated
     dice: jnp.ndarray = _roll_dice(rng1)  # diceを振る
     playable_dice: jnp.ndarray = _set_playable_dice(dice)  # play可能なサイコロを初期化
     played_dice_num: jnp.ndarray = jnp.int16(0)
@@ -569,13 +567,17 @@ def _remains_at_inner(board: jnp.ndarray, turn: jnp.ndarray) -> bool:
 def _legal_action_mask(
     board: jnp.ndarray, turn: jnp.ndarray, dice: jnp.ndarray
 ) -> jnp.ndarray:
-
+    no_op_mask = jnp.zeros(26 * 6 + 6, dtype=jnp.bool_).at[0:6].set(TRUE)
     legal_action_mask = jax.vmap(
         partial(_legal_action_mask_for_single_die, board=board, turn=turn)
     )(die=dice).any(
         axis=0
     )  # (26*6 + 6)
-    return legal_action_mask
+    legal_action_exists = ~(legal_action_mask.sum() == 0)
+    return (
+        legal_action_exists * legal_action_mask
+        + ~legal_action_exists * no_op_mask
+    )  # legal_actionがなければ, np_op maskを返す
 
 
 def _legal_action_mask_for_single_die(
@@ -584,7 +586,7 @@ def _legal_action_mask_for_single_die(
     """
     一つのサイコロの目に対するlegal micro action
     """
-    return (die == -1) * jnp.zeros(26 * 6 + 6, dtype=jnp.int16) + (
+    return (die == -1) * jnp.zeros(26 * 6 + 6, dtype=jnp.bool_) + (
         die != -1
     ) * _legal_action_mask_for_valid_single_dice(board, turn, die)
 
