@@ -1,3 +1,5 @@
+from typing import Tuple
+
 import jax
 import jax.numpy as jnp
 
@@ -7,28 +9,42 @@ from pgx.flax.struct import dataclass
 FALSE = jnp.bool_(False)
 TRUE = jnp.bool_(True)
 
+# fmt off
+IDX = jnp.int8(
+    [
+        [0, 1, 2],
+        [3, 4, 5],
+        [6, 7, 8],
+        [0, 3, 6],
+        [1, 4, 7],
+        [2, 5, 8],
+        [0, 4, 8],
+        [2, 4, 6],
+    ]
+)
+# fmt on
+
 
 @dataclass
 class State(core.State):
     curr_player: jnp.ndarray = jnp.int8(0)
+    observation: jnp.ndarray = jnp.zeros(27, dtype=jnp.bool_)
     reward: jnp.ndarray = jnp.float32([0.0, 0.0])
     terminated: jnp.ndarray = FALSE
     legal_action_mask: jnp.ndarray = jnp.ones(9, dtype=jnp.bool_)
-    # 0: 先手, 1: 後手
     turn: jnp.ndarray = jnp.int8(0)
     # 0 1 2
     # 3 4 5
     # 6 7 8
-    # -1: empty, 0: 先手, 1: 後手
-    board: jnp.ndarray = -jnp.ones(9, jnp.int8)
+    board: jnp.ndarray = -jnp.ones(9, jnp.int8)  # -1 (empty), 0, 1
 
 
 class TicTacToe(core.Env):
     def __init__(self):
         super().__init__()
 
-    def init(self, rng: jax.random.KeyArray) -> State:
-        return init(rng)
+    def _init(self, key: jax.random.KeyArray) -> State:
+        return init(key)
 
     def _step(self, state: core.State, action: jnp.ndarray) -> State:
         assert isinstance(state, State)
@@ -40,9 +56,14 @@ class TicTacToe(core.Env):
         assert isinstance(state, State)
         return observe(state, player_id)
 
+    @property
     def num_players(self) -> int:
         return 2
 
+    @property
+    def reward_range(self) -> Tuple[float, float]:
+        return -1.0, 1.0
+
 
 def init(rng: jax.random.KeyArray) -> State:
     rng, subkey = jax.random.split(rng)
@@ -51,9 +72,6 @@ def init(rng: jax.random.KeyArray) -> State:
 
 
 def step(state: State, action: jnp.ndarray) -> State:
-    # TODO(sotetsuk): illegal action check
-    # if state.legal_action_mask.at[action]:
-    #     ...
     board = state.board.at[action].set(state.turn)
     won = _win_check(board, state.turn)
     reward = jax.lax.cond(
@@ -64,11 +82,6 @@ def step(state: State, action: jnp.ndarray) -> State:
     terminated = won | jnp.all(board != -1)
     curr_player = (state.curr_player + 1) % 2
     legal_action_mask = board < 0
-    legal_action_mask = jax.lax.cond(
-        terminated,
-        lambda: jnp.zeros_like(legal_action_mask),
-        lambda: legal_action_mask,
-    )
     return State(
         curr_player=curr_player,
         legal_action_mask=legal_action_mask,
@@ -80,32 +93,7 @@ def step(state: State, action: jnp.ndarray) -> State:
 
 
 def _win_check(board, turn) -> jnp.ndarray:
-    # board:
-    #   0 1 2
-    #   3 4 5
-    #   6 7 8
-    won = FALSE
-    for i in range(0, 9, 3):
-        # e.g., [0, 1, 2]
-        won = jax.lax.cond(
-            jnp.all(board[i : i + 3] == turn), lambda: TRUE, lambda: won
-        )
-    for i in range(3):
-        # e.g., [0, 3, 6]
-        won = jax.lax.cond(
-            jnp.all(board[i:9:3] == turn), lambda: TRUE, lambda: won
-        )
-    won = jax.lax.cond(
-        jnp.all(board[jnp.int8([0, 4, 8])] == turn),
-        lambda: TRUE,
-        lambda: won,
-    )
-    won = jax.lax.cond(
-        jnp.all(board[jnp.int8([2, 4, 6])] == turn),
-        lambda: TRUE,
-        lambda: won,
-    )
-    return won
+    return ((board[IDX] == turn).all(axis=1)).any()
 
 
 def observe(state: State, player_id: jnp.ndarray) -> jnp.ndarray:
@@ -117,5 +105,5 @@ def observe(state: State, player_id: jnp.ndarray) -> jnp.ndarray:
     )
     return jnp.concatenate(
         [empty_board, my_board, opp_obard],
-        dtype=jnp.float16,
+        dtype=jnp.bool_,
     )
