@@ -6,17 +6,28 @@ import numpy as np
 from pgx.contractbridgebidding import (
     ContractBridgeBiddingState,
     _calculate_dds_tricks,
+    _is_partner,
     _key_to_hand,
     _load_sample_hash,
     _pbn_to_key,
+    _player_position,
+    _shuffle_players,
     _state_to_key,
     _state_to_pbn,
     _to_binary,
+    duplicate,
     init,
     step,
 )
 
 
+def test_shuffle_players():
+    for i in range(100):
+        shuffled_players = _shuffle_players()
+        assert (shuffled_players[0] - shuffled_players[2]) % 2
+        assert (shuffled_players[1] - shuffled_players[3]) % 2
+
+
 def test_init():
     curr_player, state = init()
     assert state.last_bid == -1
@@ -24,12 +35,36 @@ def test_init():
     assert not state.call_x
     assert not state.call_xx
     assert not state.pass_num
-    assert state.curr_player == state.dealer
+    assert _player_position(state.curr_player, state) == state.dealer
     assert state.legal_action_mask[:-2].all()
     assert not state.legal_action_mask[-2:].all()
 
 
+def test_duplicate():
+    for i in range(1000):
+        _, init_state = init()
+        duplicated_state = duplicate(init_state)
+        assert (
+            init_state.shuffled_players[0]
+            == duplicated_state.shuffled_players[1]
+        )
+        assert (
+            init_state.shuffled_players[1]
+            == duplicated_state.shuffled_players[0]
+        )
+        assert (
+            init_state.shuffled_players[2]
+            == duplicated_state.shuffled_players[3]
+        )
+        assert (
+            init_state.shuffled_players[3]
+            == duplicated_state.shuffled_players[2]
+        )
+
+
 def test_step():
+    #  player_id: 0 = N, 1 = S, 2 = W, 3 = E
+    #   0  3  1  2
     #   N  E  S  W
     #  -----------
     #      P  P  P
@@ -39,28 +74,31 @@ def test_step():
     #  XX  P 7C  P
     #   P  P
     _, state = init()
-    state.dealer[0] = 1
-    state.curr_player[0] = 1
-
+    state.dealer = 1
+    state.curr_player = 3
+    state.shuffled_players = np.array([0, 3, 1, 2], dtype=np.int8)
     bidding_history = np.full(319, -1, dtype=np.int8)
-    legal_action_mask = np.ones(38, dtype=np.bool8)
+    legal_action_mask = np.ones(38, dtype=np.bool_)
     legal_action_mask[-2:] = 0
     first_denomination_NS = np.full(5, -1, dtype=np.int8)
     first_denomination_EW = np.full(5, -1, dtype=np.int8)
 
     curr_player, state, rewards = step(state, 35)
+    #  player_id: 0 = N, 1 = S, 2 = W, 3 = E
+    #   0  3  1  2
     #   N  E  S  W
     #  -----------
     #      P
 
     assert state.turn == 1
-    assert curr_player == 2
+    assert state.curr_player == 1
+    assert _player_position(curr_player, state) == 2
     assert not state.terminated
     bidding_history[0] = 35
     assert np.all(state.bidding_history == bidding_history)
     assert np.all(state.legal_action_mask == legal_action_mask)
     assert state.last_bid == -1
-    assert state.last_bidder == -1
+    assert _player_position(state.last_bidder, state) == -1
     assert not state.call_x
     assert not state.call_xx
     assert np.all(state.first_denomination_NS == first_denomination_NS)
@@ -68,18 +106,21 @@ def test_step():
     assert state.pass_num == 1
 
     curr_player, state, rewards = step(state, 35)
+    #  player_id: 0 = N, 1 = S, 2 = W, 3 = E
+    #   0  3  1  2
     #   N  E  S  W
     #  -----------
     #      P  P
 
     assert state.turn == 2
-    assert curr_player == 3
+    assert state.curr_player == 2
+    assert _player_position(curr_player, state) == 3
     assert not state.terminated
     bidding_history[1] = 35
     assert np.all(state.bidding_history == bidding_history)
     assert np.all(state.legal_action_mask == legal_action_mask)
     assert state.last_bid == -1
-    assert state.last_bidder == -1
+    assert _player_position(state.last_bidder, state) == -1
     assert not state.call_x
     assert not state.call_xx
     assert np.all(state.first_denomination_NS == first_denomination_NS)
@@ -87,18 +128,21 @@ def test_step():
     assert state.pass_num == 2
 
     curr_player, state, rewards = step(state, 35)
+    #  player_id: 0 = N, 1 = S, 2 = W, 3 = E
+    #   0  3  1  2
     #   N  E  S  W
     #  -----------
     #      P  P  P
 
     assert state.turn == 3
-    assert curr_player == 0
+    assert state.curr_player == 0
+    assert _player_position(curr_player, state) == 0
     assert not state.terminated
     bidding_history[2] = 35
     assert np.all(state.bidding_history == bidding_history)
     assert np.all(state.legal_action_mask == legal_action_mask)
     assert state.last_bid == -1
-    assert state.last_bidder == -1
+    assert _player_position(state.last_bidder, state) == -1
     assert not state.call_x
     assert not state.call_xx
     assert np.all(state.first_denomination_NS == first_denomination_NS)
@@ -106,6 +150,8 @@ def test_step():
     assert state.pass_num == 3
 
     curr_player, state, rewards = step(state, 0)
+    #  player_id: 0 = N, 1 = S, 2 = W, 3 = E
+    #   0  3  1  2
     #   N  E  S  W
     #  -----------
     #      P  P  P
@@ -114,7 +160,8 @@ def test_step():
     first_denomination_EW = np.array([-1, -1, -1, -1, -1])
 
     assert state.turn == 4
-    assert curr_player == 1
+    assert state.curr_player == 3
+    assert _player_position(curr_player, state) == 1
     assert not state.terminated
     bidding_history[3] = 0
     assert np.all(state.bidding_history == bidding_history)
@@ -122,7 +169,7 @@ def test_step():
     legal_action_mask[36] = 1
     assert np.all(state.legal_action_mask == legal_action_mask)
     assert state.last_bid == 0
-    assert state.last_bidder == 0
+    assert _player_position(state.last_bidder, state) == 0
     assert not state.call_x
     assert not state.call_xx
     assert np.all(state.first_denomination_NS == first_denomination_NS)
@@ -130,23 +177,27 @@ def test_step():
     assert state.pass_num == 0
 
     curr_player, state, rewards = step(state, 8)
+    #  player_id: 0 = N, 1 = S, 2 = W, 3 = E
+    #   0  3  1  2
     #   N  E  S  W
     #  -----------
     #      P  P  P
     #  1C 2S
     first_denomination_NS = np.array([0, -1, -1, -1, -1])
-    first_denomination_EW = np.array([-1, -1, -1, 1, -1])
+    first_denomination_EW = np.array([-1, -1, -1, 3, -1])
 
     assert state.turn == 5
-    assert curr_player == 2
+    assert state.curr_player == 1
+    assert _player_position(curr_player, state) == 2
     assert not state.terminated
     bidding_history[4] = 8
     assert np.all(state.bidding_history == bidding_history)
     legal_action_mask[0:9] = 0
     legal_action_mask[36] = 1
+
     assert np.all(state.legal_action_mask == legal_action_mask)
     assert state.last_bid == 8
-    assert state.last_bidder == 1
+    assert _player_position(state.last_bidder, state) == 1
     assert not state.call_x
     assert not state.call_xx
     assert np.all(state.first_denomination_NS == first_denomination_NS)
@@ -154,15 +205,18 @@ def test_step():
     assert state.pass_num == 0
 
     curr_player, state, rewards = step(state, 36)
+    #  player_id: 0 = N, 1 = S, 2 = W, 3 = E
+    #   0  3  1  2
     #   N  E  S  W
     #  -----------
     #      P  P  P
     #  1C 2S  X
     first_denomination_NS = np.array([0, -1, -1, -1, -1])
-    first_denomination_EW = np.array([-1, -1, -1, 1, -1])
+    first_denomination_EW = np.array([-1, -1, -1, 3, -1])
 
     assert state.turn == 6
-    assert curr_player == 3
+    assert state.curr_player == 2
+    assert _player_position(curr_player, state) == 3
     assert not state.terminated
     bidding_history[5] = 36
     assert np.all(state.bidding_history == bidding_history)
@@ -171,7 +225,7 @@ def test_step():
     legal_action_mask[37] = 1
     assert np.all(state.legal_action_mask == legal_action_mask)
     assert state.last_bid == 8
-    assert state.last_bidder == 1
+    assert _player_position(state.last_bidder, state) == 1
     assert state.call_x
     assert not state.call_xx
     assert np.all(state.first_denomination_NS == first_denomination_NS)
@@ -179,25 +233,28 @@ def test_step():
     assert state.pass_num == 0
 
     curr_player, state, rewards = step(state, 35)
+    #  player_id: 0 = N, 1 = S, 2 = W, 3 = E
+    #   0  3  1  2
     #   N  E  S  W
     #  -----------
     #      P  P  P
     #  1C 2S  X  P
     first_denomination_NS = np.array([0, -1, -1, -1, -1])
-    first_denomination_EW = np.array([-1, -1, -1, 1, -1])
+    first_denomination_EW = np.array([-1, -1, -1, 3, -1])
 
     assert state.turn == 7
-    assert curr_player == 0
+    assert state.curr_player == 0
+    assert _player_position(curr_player, state) == 0
     assert not state.terminated
     bidding_history[6] = 35
     assert np.all(state.bidding_history == bidding_history)
-    legal_action_mask = np.ones(38, dtype=np.bool8)
+    legal_action_mask = np.ones(38, dtype=np.bool_)
     legal_action_mask[0:9] = 0
     legal_action_mask[36] = 0
     legal_action_mask[37] = 0
     assert np.all(state.legal_action_mask == legal_action_mask)
     assert state.last_bid == 8
-    assert state.last_bidder == 1
+    assert _player_position(state.last_bidder, state) == 1
     assert state.call_x
     assert not state.call_xx
     assert np.all(state.first_denomination_NS == first_denomination_NS)
@@ -205,26 +262,29 @@ def test_step():
     assert state.pass_num == 1
 
     curr_player, state, rewards = step(state, 35)
+    #  player_id: 0 = N, 1 = S, 2 = W, 3 = E
+    #   0  3  1  2
     #   N  E  S  W
     #  -----------
     #      P  P  P
     #  1C 2S  X  P
     #   P
     first_denomination_NS = np.array([0, -1, -1, -1, -1])
-    first_denomination_EW = np.array([-1, -1, -1, 1, -1])
+    first_denomination_EW = np.array([-1, -1, -1, 3, -1])
 
     assert state.turn == 8
-    assert curr_player == 1
+    assert state.curr_player == 3
+    assert _player_position(curr_player, state) == 1
     assert not state.terminated
     bidding_history[7] = 35
     assert np.all(state.bidding_history == bidding_history)
-    legal_action_mask = np.ones(38, dtype=np.bool8)
+    legal_action_mask = np.ones(38, dtype=np.bool_)
     legal_action_mask[0:9] = 0
     legal_action_mask[36] = 0
     legal_action_mask[37] = 1
     assert np.all(state.legal_action_mask == legal_action_mask)
     assert state.last_bid == 8
-    assert state.last_bidder == 1
+    assert _player_position(state.last_bidder, state) == 1
     assert state.call_x
     assert not state.call_xx
     assert np.all(state.first_denomination_NS == first_denomination_NS)
@@ -232,26 +292,29 @@ def test_step():
     assert state.pass_num == 2
 
     curr_player, state, rewards = step(state, 37)
+    #  player_id: 0 = N, 1 = S, 2 = W, 3 = E
+    #   0  3  1  2
     #   N  E  S  W
     #  -----------
     #      P  P  P
     #  1C 2S  X  P
     #   P XX
     first_denomination_NS = np.array([0, -1, -1, -1, -1])
-    first_denomination_EW = np.array([-1, -1, -1, 1, -1])
+    first_denomination_EW = np.array([-1, -1, -1, 3, -1])
 
     assert state.turn == 9
-    assert curr_player == 2
+    assert state.curr_player == 1
+    assert _player_position(curr_player, state) == 2
     assert not state.terminated
     bidding_history[8] = 37
     assert np.all(state.bidding_history == bidding_history)
-    legal_action_mask = np.ones(38, dtype=np.bool8)
+    legal_action_mask = np.ones(38, dtype=np.bool_)
     legal_action_mask[0:9] = 0
     legal_action_mask[36] = 0
     legal_action_mask[37] = 0
     assert np.all(state.legal_action_mask == legal_action_mask)
     assert state.last_bid == 8
-    assert state.last_bidder == 1
+    assert _player_position(state.last_bidder, state) == 1
     assert state.call_x
     assert state.call_xx
     assert np.all(state.first_denomination_NS == first_denomination_NS)
@@ -259,26 +322,29 @@ def test_step():
     assert state.pass_num == 0
 
     curr_player, state, rewards = step(state, 35)
+    #  player_id: 0 = N, 1 = S, 2 = W, 3 = E
+    #   0  3  1  2
     #   N  E  S  W
     #  -----------
     #      P  P  P
     #  1C 2S  X  P
     #   P XX  P
     first_denomination_NS = np.array([0, -1, -1, -1, -1])
-    first_denomination_EW = np.array([-1, -1, -1, 1, -1])
+    first_denomination_EW = np.array([-1, -1, -1, 3, -1])
 
     assert state.turn == 10
-    assert curr_player == 3
+    assert state.curr_player == 2
+    assert _player_position(curr_player, state) == 3
     assert not state.terminated
     bidding_history[9] = 35
     assert np.all(state.bidding_history == bidding_history)
-    legal_action_mask = np.ones(38, dtype=np.bool8)
+    legal_action_mask = np.ones(38, dtype=np.bool_)
     legal_action_mask[0:9] = 0
     legal_action_mask[36] = 0
     legal_action_mask[37] = 0
     assert np.all(state.legal_action_mask == legal_action_mask)
     assert state.last_bid == 8
-    assert state.last_bidder == 1
+    assert _player_position(state.last_bidder, state) == 1
     assert state.call_x
     assert state.call_xx
     assert np.all(state.first_denomination_NS == first_denomination_NS)
@@ -286,26 +352,29 @@ def test_step():
     assert state.pass_num == 1
 
     curr_player, state, rewards = step(state, 22)
+    #  player_id: 0 = N, 1 = S, 2 = W, 3 = E
+    #   0  3  1  2
     #   N  E  S  W
     #  -----------
     #      P  P  P
     #  1C 2S  X  P
     #   P XX  P 5H
     first_denomination_NS = np.array([0, -1, -1, -1, -1])
-    first_denomination_EW = np.array([-1, -1, 3, 1, -1])
+    first_denomination_EW = np.array([-1, -1, 2, 3, -1])
 
     assert state.turn == 11
-    assert curr_player == 0
+    assert state.curr_player == 0
+    assert _player_position(curr_player, state) == 0
     assert not state.terminated
     bidding_history[10] = 22
     assert np.all(state.bidding_history == bidding_history)
-    legal_action_mask = np.ones(38, dtype=np.bool8)
+    legal_action_mask = np.ones(38, dtype=np.bool_)
     legal_action_mask[0:23] = 0
     legal_action_mask[36] = 1
     legal_action_mask[37] = 0
     assert np.all(state.legal_action_mask == legal_action_mask)
     assert state.last_bid == 22
-    assert state.last_bidder == 3
+    assert _player_position(state.last_bidder, state) == 3
     assert not state.call_x
     assert not state.call_xx
     assert np.all(state.first_denomination_NS == first_denomination_NS)
@@ -313,6 +382,8 @@ def test_step():
     assert state.pass_num == 0
 
     curr_player, state, rewards = step(state, 23)
+    #  player_id: 0 = N, 1 = S, 2 = W, 3 = E
+    #   0  3  1  2
     #   N  E  S  W
     #  -----------
     #      P  P  P
@@ -320,20 +391,21 @@ def test_step():
     #   P XX  P 5H
     #  5S
     first_denomination_NS = np.array([0, -1, -1, 0, -1])
-    first_denomination_EW = np.array([-1, -1, 3, 1, -1])
+    first_denomination_EW = np.array([-1, -1, 2, 3, -1])
 
     assert state.turn == 12
-    assert curr_player == 1
+    assert state.curr_player == 3
+    assert _player_position(curr_player, state) == 1
     assert not state.terminated
     bidding_history[11] = 23
     assert np.all(state.bidding_history == bidding_history)
-    legal_action_mask = np.ones(38, dtype=np.bool8)
+    legal_action_mask = np.ones(38, dtype=np.bool_)
     legal_action_mask[0:24] = 0
     legal_action_mask[36] = 1
     legal_action_mask[37] = 0
     assert np.all(state.legal_action_mask == legal_action_mask)
     assert state.last_bid == 23
-    assert state.last_bidder == 0
+    assert _player_position(state.last_bidder, state) == 0
     assert not state.call_x
     assert not state.call_xx
     assert np.all(state.first_denomination_NS == first_denomination_NS)
@@ -341,6 +413,8 @@ def test_step():
     assert state.pass_num == 0
 
     curr_player, state, rewards = step(state, 35)
+    #  player_id: 0 = N, 1 = S, 2 = W, 3 = E
+    #   0  3  1  2
     #   N  E  S  W
     #  -----------
     #      P  P  P
@@ -348,20 +422,21 @@ def test_step():
     #   P XX  P 5H
     #  5S  P
     first_denomination_NS = np.array([0, -1, -1, 0, -1])
-    first_denomination_EW = np.array([-1, -1, 3, 1, -1])
+    first_denomination_EW = np.array([-1, -1, 2, 3, -1])
 
     assert state.turn == 13
-    assert curr_player == 2
+    assert state.curr_player == 1
+    assert _player_position(curr_player, state) == 2
     assert not state.terminated
     bidding_history[12] = 35
     assert np.all(state.bidding_history == bidding_history)
-    legal_action_mask = np.ones(38, dtype=np.bool8)
+    legal_action_mask = np.ones(38, dtype=np.bool_)
     legal_action_mask[0:24] = 0
     legal_action_mask[36] = 0
     legal_action_mask[37] = 0
     assert np.all(state.legal_action_mask == legal_action_mask)
     assert state.last_bid == 23
-    assert state.last_bidder == 0
+    assert _player_position(state.last_bidder, state) == 0
     assert not state.call_x
     assert not state.call_xx
     assert np.all(state.first_denomination_NS == first_denomination_NS)
@@ -369,6 +444,8 @@ def test_step():
     assert state.pass_num == 1
 
     curr_player, state, rewards = step(state, 25)
+    #  player_id: 0 = N, 1 = S, 2 = W, 3 = E
+    #   0  3  1  2
     #   N  E  S  W
     #  -----------
     #      P  P  P
@@ -376,20 +453,21 @@ def test_step():
     #   P XX  P 5H
     #  5S  P 6C
     first_denomination_NS = np.array([0, -1, -1, 0, -1])
-    first_denomination_EW = np.array([-1, -1, 3, 1, -1])
+    first_denomination_EW = np.array([-1, -1, 2, 3, -1])
 
     assert state.turn == 14
-    assert curr_player == 3
+    assert state.curr_player == 2
+    assert _player_position(curr_player, state) == 3
     assert not state.terminated
     bidding_history[13] = 25
     assert np.all(state.bidding_history == bidding_history)
-    legal_action_mask = np.ones(38, dtype=np.bool8)
+    legal_action_mask = np.ones(38, dtype=np.bool_)
     legal_action_mask[0:26] = 0
     legal_action_mask[36] = 1
     legal_action_mask[37] = 0
     assert np.all(state.legal_action_mask == legal_action_mask)
     assert state.last_bid == 25
-    assert state.last_bidder == 2
+    assert _player_position(state.last_bidder, state) == 2
     assert not state.call_x
     assert not state.call_xx
     assert np.all(state.first_denomination_NS == first_denomination_NS)
@@ -397,6 +475,8 @@ def test_step():
     assert state.pass_num == 0
 
     curr_player, state, rewards = step(state, 36)
+    #  player_id: 0 = N, 1 = S, 2 = W, 3 = E
+    #   0  3  1  2
     #   N  E  S  W
     #  -----------
     #      P  P  P
@@ -404,20 +484,21 @@ def test_step():
     #   P XX  P 5H
     #  5S  P 6C  X
     first_denomination_NS = np.array([0, -1, -1, 0, -1])
-    first_denomination_EW = np.array([-1, -1, 3, 1, -1])
+    first_denomination_EW = np.array([-1, -1, 2, 3, -1])
 
     assert state.turn == 15
-    assert curr_player == 0
+    assert state.curr_player == 0
+    assert _player_position(curr_player, state) == 0
     assert not state.terminated
     bidding_history[14] = 36
     assert np.all(state.bidding_history == bidding_history)
-    legal_action_mask = np.ones(38, dtype=np.bool8)
+    legal_action_mask = np.ones(38, dtype=np.bool_)
     legal_action_mask[0:26] = 0
     legal_action_mask[36] = 0
     legal_action_mask[37] = 1
     assert np.all(state.legal_action_mask == legal_action_mask)
     assert state.last_bid == 25
-    assert state.last_bidder == 2
+    assert _player_position(state.last_bidder, state) == 2
     assert state.call_x
     assert not state.call_xx
     assert np.all(state.first_denomination_NS == first_denomination_NS)
@@ -425,6 +506,8 @@ def test_step():
     assert state.pass_num == 0
 
     curr_player, state, rewards = step(state, 37)
+    #  player_id: 0 = N, 1 = S, 2 = W, 3 = E
+    #   0  3  1  2
     #   N  E  S  W
     #  -----------
     #      P  P  P
@@ -433,20 +516,21 @@ def test_step():
     #  5S  P 6C  X
     #  XX
     first_denomination_NS = np.array([0, -1, -1, 0, -1])
-    first_denomination_EW = np.array([-1, -1, 3, 1, -1])
+    first_denomination_EW = np.array([-1, -1, 2, 3, -1])
 
     assert state.turn == 16
-    assert curr_player == 1
+    assert state.curr_player == 3
+    assert _player_position(curr_player, state) == 1
     assert not state.terminated
     bidding_history[15] = 37
     assert np.all(state.bidding_history == bidding_history)
-    legal_action_mask = np.ones(38, dtype=np.bool8)
+    legal_action_mask = np.ones(38, dtype=np.bool_)
     legal_action_mask[0:26] = 0
     legal_action_mask[36] = 0
     legal_action_mask[37] = 0
     assert np.all(state.legal_action_mask == legal_action_mask)
     assert state.last_bid == 25
-    assert state.last_bidder == 2
+    assert _player_position(state.last_bidder, state) == 2
     assert state.call_x
     assert state.call_xx
     assert np.all(state.first_denomination_NS == first_denomination_NS)
@@ -454,6 +538,8 @@ def test_step():
     assert state.pass_num == 0
 
     curr_player, state, rewards = step(state, 35)
+    #  player_id: 0 = N, 1 = S, 2 = W, 3 = E
+    #   0  3  1  2
     #   N  E  S  W
     #  -----------
     #      P  P  P
@@ -462,20 +548,21 @@ def test_step():
     #  5S  P 6C  X
     #  XX  P
     first_denomination_NS = np.array([0, -1, -1, 0, -1])
-    first_denomination_EW = np.array([-1, -1, 3, 1, -1])
+    first_denomination_EW = np.array([-1, -1, 2, 3, -1])
 
     assert state.turn == 17
-    assert curr_player == 2
+    assert state.curr_player == 1
+    assert _player_position(curr_player, state) == 2
     assert not state.terminated
     bidding_history[16] = 35
     assert np.all(state.bidding_history == bidding_history)
-    legal_action_mask = np.ones(38, dtype=np.bool8)
+    legal_action_mask = np.ones(38, dtype=np.bool_)
     legal_action_mask[0:26] = 0
     legal_action_mask[36] = 0
     legal_action_mask[37] = 0
     assert np.all(state.legal_action_mask == legal_action_mask)
     assert state.last_bid == 25
-    assert state.last_bidder == 2
+    assert _player_position(state.last_bidder, state) == 2
     assert state.call_x
     assert state.call_xx
     assert np.all(state.first_denomination_NS == first_denomination_NS)
@@ -483,6 +570,8 @@ def test_step():
     assert state.pass_num == 1
 
     curr_player, state, rewards = step(state, 30)
+    #  player_id: 0 = N, 1 = S, 2 = W, 3 = E
+    #   0  3  1  2
     #   N  E  S  W
     #  -----------
     #      P  P  P
@@ -491,20 +580,21 @@ def test_step():
     #  5S  P 6C  X
     #  XX  P 7C
     first_denomination_NS = np.array([0, -1, -1, 0, -1])
-    first_denomination_EW = np.array([-1, -1, 3, 1, -1])
+    first_denomination_EW = np.array([-1, -1, 2, 3, -1])
 
     assert state.turn == 18
-    assert curr_player == 3
+    assert state.curr_player == 2
+    assert _player_position(curr_player, state) == 3
     assert not state.terminated
     bidding_history[17] = 30
     assert np.all(state.bidding_history == bidding_history)
-    legal_action_mask = np.ones(38, dtype=np.bool8)
+    legal_action_mask = np.ones(38, dtype=np.bool_)
     legal_action_mask[0:31] = 0
     legal_action_mask[36] = 1
     legal_action_mask[37] = 0
     assert np.all(state.legal_action_mask == legal_action_mask)
     assert state.last_bid == 30
-    assert state.last_bidder == 2
+    assert _player_position(state.last_bidder, state) == 2
     assert not state.call_x
     assert not state.call_xx
     assert np.all(state.first_denomination_NS == first_denomination_NS)
@@ -512,6 +602,8 @@ def test_step():
     assert state.pass_num == 0
 
     curr_player, state, rewards = step(state, 35)
+    #  player_id: 0 = N, 1 = S, 2 = W, 3 = E
+    #   0  3  1  2
     #   N  E  S  W
     #  -----------
     #      P  P  P
@@ -520,20 +612,21 @@ def test_step():
     #  5S  P 6C  X
     #  XX  P 7C  P
     first_denomination_NS = np.array([0, -1, -1, 0, -1])
-    first_denomination_EW = np.array([-1, -1, 3, 1, -1])
+    first_denomination_EW = np.array([-1, -1, 2, 3, -1])
 
     assert state.turn == 19
-    assert curr_player == 0
+    assert state.curr_player == 0
+    assert _player_position(curr_player, state) == 0
     assert not state.terminated
     bidding_history[18] = 35
     assert np.all(state.bidding_history == bidding_history)
-    legal_action_mask = np.ones(38, dtype=np.bool8)
+    legal_action_mask = np.ones(38, dtype=np.bool_)
     legal_action_mask[0:31] = 0
     legal_action_mask[36] = 0
     legal_action_mask[37] = 0
     assert np.all(state.legal_action_mask == legal_action_mask)
     assert state.last_bid == 30
-    assert state.last_bidder == 2
+    assert _player_position(state.last_bidder, state) == 2
     assert not state.call_x
     assert not state.call_xx
     assert np.all(state.first_denomination_NS == first_denomination_NS)
@@ -541,6 +634,8 @@ def test_step():
     assert state.pass_num == 1
 
     curr_player, state, rewards = step(state, 35)
+    #  player_id: 0 = N, 1 = S, 2 = W, 3 = E
+    #   0  3  1  2
     #   N  E  S  W
     #  -----------
     #      P  P  P
@@ -550,20 +645,21 @@ def test_step():
     #  XX  P 7C  P
     #   P
     first_denomination_NS = np.array([0, -1, -1, 0, -1])
-    first_denomination_EW = np.array([-1, -1, 3, 1, -1])
+    first_denomination_EW = np.array([-1, -1, 2, 3, -1])
 
     assert state.turn == 20
-    assert curr_player == 1
+    assert state.curr_player == 3
+    assert _player_position(curr_player, state) == 1
     assert not state.terminated
     bidding_history[19] = 35
     assert np.all(state.bidding_history == bidding_history)
-    legal_action_mask = np.ones(38, dtype=np.bool8)
+    legal_action_mask = np.ones(38, dtype=np.bool_)
     legal_action_mask[0:31] = 0
     legal_action_mask[36] = 1
     legal_action_mask[37] = 0
     assert np.all(state.legal_action_mask == legal_action_mask)
     assert state.last_bid == 30
-    assert state.last_bidder == 2
+    assert _player_position(state.last_bidder, state) == 2
     assert not state.call_x
     assert not state.call_xx
     assert np.all(state.first_denomination_NS == first_denomination_NS)
@@ -571,6 +667,8 @@ def test_step():
     assert state.pass_num == 2
 
     curr_player, state, rewards = step(state, 35)
+    #  player_id: 0 = N, 1 = S, 2 = W, 3 = E
+    #   0  3  1  2
     #   N  E  S  W
     #  -----------
     #      P  P  P
@@ -580,23 +678,23 @@ def test_step():
     #  XX  P 7C  P
     #   P  P
     # Passが3回続いたので終了
-    # state.terminated = True
-    # state.curr_player = -1
+    state.terminated = True
+    state.curr_player = -1
     first_denomination_NS = np.array([0, -1, -1, 0, -1])
-    first_denomination_EW = np.array([-1, -1, 3, 1, -1])
+    first_denomination_EW = np.array([-1, -1, 2, 3, -1])
 
     assert state.turn == 20
-    assert curr_player == -1
+    assert _player_position(curr_player, state) == -1
     assert state.terminated
     bidding_history[20] = 35
     assert np.all(state.bidding_history == bidding_history)
-    legal_action_mask = np.ones(38, dtype=np.bool8)
+    legal_action_mask = np.ones(38, dtype=np.bool_)
     legal_action_mask[0:31] = 0
     legal_action_mask[36] = 1
     legal_action_mask[37] = 0
     assert np.all(state.legal_action_mask == legal_action_mask)
     assert state.last_bid == 30
-    assert state.last_bidder == 2
+    assert _player_position(state.last_bidder, state) == 2
     assert not state.call_x
     assert not state.call_xx
     assert np.all(state.first_denomination_NS == first_denomination_NS)
@@ -620,15 +718,15 @@ def max_action_length_agent(state: ContractBridgeBiddingState):
 def test_max_action():
     _, state = init()
     state.turn = np.int16(0)
-    state.terminated = np.bool8(0)
+    state.terminated = np.bool_(0)
     state.bidding_history = np.full(319, -1, dtype=np.int8)
-    state.legal_action_mask = np.ones(38, dtype=np.bool8)
+    state.legal_action_mask = np.ones(38, dtype=np.bool_)
     state.legal_action_mask[-2:] = 0
     state.first_denomination_NS = np.full(5, -1, dtype=np.int8)
     state.first_denomination_EW = np.full(5, -1, dtype=np.int8)
-    state.call_x = np.bool8(0)
-    state.call_xx = np.bool8(0)
-    state.pass_num = np.bool8(0)
+    state.call_x = np.bool_(0)
+    state.call_xx = np.bool_(0)
+    state.pass_num = np.bool_(0)
     state.last_bid = np.int8(-1)
     state.last_bidder = np.int8(-1)
 
